var APILogs = (function () {
  var Service = null;
  var APILogsData = [];
  var APILogsEditor = '';
  var currentCursorPosition = false;

  var FeederLogData = [];
  var FeederLogEditor = '';
  var FeederAppConfigData = [];
  var FeederAppConfigEditor = '';
  var FeederHostConfigData = [];
  var FeederHostConfigEditor = '';

  var currentCursorPositionFeeder = false;
  function init() {
    let element = $('#dvPageContent')
    element.on('click', '#APILogsEditorSearch', function () {
      APILogsEditor.execCommand('find');
    });
    element.on('click', '#APILogsDownload', function () {
      downloadLogs();
    });

    element.on('click', '#PTFConfigEditorAppDownload', function () {
      downloadPTFLogs('APP');
    });
    element.on('click', '#PTFConfigEditorAppSearch', function () {
      FeederAppConfigEditor.execCommand('find');
    });
    element.on('click', '#PTFConfigEditorHostDownload', function () {
      downloadPTFLogs('HOST');
    });
    element.on('click', '#PTFConfigEditorHostSearch', function () {
      FeederHostConfigEditor.execCommand('find');
    });
    element.on('click', '#PTFLogDownload', function () {
      downloadPTFLogs('LOGS');
    });
    element.on('click', '#PTFLogSearch', function () {
      FeederLogEditor.execCommand('find');
    });

    Service = Services(
      PBConstants.BASE_API_URL,
      PBConstants.MARKET_CAP_API_URL,
      PBConstants.CURRENCIES_API_URL,
      PBConstants.GLOBAL_MARKET_CAP_API_URL
    );

    handleWindowResize();
  }

  function handleWindowResize() {
    var resized;
    $(window).on('resize orientationChanged', function () {
      clearTimeout(resized);
      resized = setTimeout(function () {
        if (APILogsEditor) {
          APILogsEditor.resize();
        }
        if(FeederLogEditor) {
          FeederLogEditor.resize();
        }
      }, 400);
    });
  }

  function downloadLogs() {
    var tmpLink = '<a class="logs-download hide" href="' + PBConstants.BASE_API_URL + '/api/v2/errors/log/zipped"></a>';
    $('body').append(tmpLink);
    var $logsDownloadLink = $('.logs-download');
    $logsDownloadLink[0].click();
    $logsDownloadLink.remove();
  }

  function downloadPTFLogs(area) {
    if(area === 'APP' || area === 'HOST') {
      var tmpLink = '<a class="logs-download hide" href="' + PBConstants.BASE_API_URL + '/settings/feeder/config/zipped"></a>';
      $('body').append(tmpLink);
      var $logsDownloadLink = $('.logs-download');
      $logsDownloadLink[0].click();
      $logsDownloadLink.remove();
    }

    if(area === 'LOGS') {
      tmpLink = '<a class="logs-download hide" href="' + PBConstants.BASE_API_URL + '/api/v2/feeder/log/zipped"></a>';
      $('body').append(tmpLink);
      var $logsDownloadLink = $('.logs-download');
      $logsDownloadLink[0].click();
      $logsDownloadLink.remove();
    }
  }

  function callAPILogs() {
    $('#APILogsEditorLoading').removeClass('visibility-hidden').addClass('visibility-show');
    // If API logs call fails then ignore and show previous data.
    Service.getAPILogs().done(function (response) {
      normalizeAPILogsData(response);
      $('#APILogsEditorLoading').removeClass('visibility-show').addClass('visibility-hidden');
      currentCursorPosition = true;
    });
    // Call API logs for every x seconds.
    setTimeout(callAPILogs, PBConstants.API_LOGS_REFRESH);
  }

  function normalizeAPILogsData(currentAPILogsData) {
    // If API response is not array then show previous data.
    if ($.isArray(currentAPILogsData)) {
      APILogsData = currentAPILogsData;
    }
    setAPILogsContent(APILogsEditor, currentCursorPosition);
  }

  function setAPILogsContent(editor, currentCursorPositionParam) {
    if(!editor) {
      return;
    }
    APILogsEditor = editor;
    let APILogsDataString = APILogsData.join('\n');
    var editorScrollTop = APILogsEditor.session.getScrollTop();
    APILogsEditor.session.setValue(APILogsDataString, 1);
    var EditorLength = APILogsEditor.session.getLength(); 
    // Logs editor should be readonly.
    APILogsEditor.setReadOnly(true);

    // Scrolling to end of the editor on page load and also when
    // user switches from another tab to Logs tab.
    // Not scrolling after API call to improve readability.
    if (currentCursorPositionParam) {
      APILogsEditor.session.setScrollTop(editorScrollTop + EditorLength);
    } else {
      // Ace eidtor gotoLine is not working in lower resolution so scrolling to bottom.
      if ($(window).innerWidth() < PBConstants.MIN_RESOLUTION_FOR_FIXED_HEADERS) {
        setTimeout(function () {
          $('html, body').animate({
            scrollTop: $('.footer').offset().top
          });
        }, 500);
      } else {
        var count = 0;
        APILogsEditor.gotoLine(EditorLength);
        $(document).on("click", 'a[href="#logs"]', function(e) {
          var scroll = setInterval(function() {
            var row = editor.session.getLength() - 1;
            var column = editor.session.getLine(row).length;
            var value = row * column;
            APILogsEditor.session.setScrollTop(EditorLength + value);
            if (count === PBConstants.MAX_ITERATIONS) {
              clearInterval(scroll);
            }
            count++;
          }, 100);
        });
      }

    }
  }

  function callPTFeederLogs() {
    $('#APILogsEditorLoadingFeeder').removeClass('visibility-hidden').addClass('visibility-show');
    Service.getPTFeederLogs().done(function (response) {
      normalizePTFeederLog(response);
      $('#APILogsEditorLoadingFeeder').removeClass('visibility-show').addClass('visibility-hidden');
      currentCursorPositionFeeder = true;
    });
    setTimeout(callPTFeederLogs, PBConstants.API_LOGS_REFRESH);
  }

  function normalizePTFeederLog(currentLogData) {
    if($.isArray(currentLogData)) {
      FeederLogData = currentLogData;
    }
    setPTFeederContent(FeederLogEditor, currentCursorPositionFeeder)
  }

  function setPTFeederContent(editor, currentCursorPositionParam) {
    if(!editor) {
      return;
    }
    FeederLogEditor = editor;
    let feederDataString = FeederLogData.join('\n');
    var editorScrollTop = FeederLogEditor.session.getScrollTop();
    FeederLogEditor.session.setValue(feederDataString, 1);
    // Logs editor should be readonly.
    FeederLogEditor.setReadOnly(true);

    // Scrolling to end of the editor on page load and also when
    // user switches from another tab to Logs tab.
    // Not scrolling after API call to improve readability.
    if (currentCursorPositionParam) {
      FeederLogEditor.session.setScrollTop(editorScrollTop);
    } else {
    //   // Ace eidtor gotoLine is not working in lower resolution so scrolling to bottom.
      if ($(window).innerWidth() < PBConstants.MIN_RESOLUTION_FOR_FIXED_HEADERS) {
        setTimeout(function () {
          $('html, body').animate({
            scrollTop: $('.footer').offset().top
          });
        }, 500);
      } else {
        setTimeout(function () {
          var EditorLength = FeederLogEditor.session.getLength();
          FeederLogEditor.gotoLine(EditorLength);
        }, 100);
      }

    }
  }

  function callPTFeederAppConfig() {
    $('#PTFConfigEditorAppLoading').removeClass('visibility-hidden').addClass('visibility-show');
    Service.getConfigFile('PTFEEDER', 'APP').done(function (response) {
      normalizePTFeederAppConfig(response);
      $('#PTFConfigEditorAppLoading').removeClass('visibility-show').addClass('visibility-hidden');
    });
  }

  function normalizePTFeederAppConfig(currentLogData) {
    FeederAppConfigData = currentLogData;
    setPTFeederAppConfig(FeederAppConfigEditor)
  }

  function setPTFeederAppConfig(editor) {

    if(!editor) {
      return;
    }
    FeederAppConfigEditor = editor;
    if (FeederAppConfigData && FeederAppConfigData.length > 0){
      FeederAppConfigEditor.session.setValue(FeederAppConfigData, 1);
    }
  }

  function callPTFeederHostConfig() {
    $('#PTFConfigEditorHostLoading').removeClass('visibility-hidden').addClass('visibility-show');
    Service.getConfigFile('PTFEEDER', 'HOST').done(function (response) {
      normalizePTFeederHostConfig(response);
      $('#PTFConfigEditorHostLoading').removeClass('visibility-show').addClass('visibility-hidden');
    });
  }

  function normalizePTFeederHostConfig(currentLogData) {
    FeederHostConfigData = currentLogData;
    setPTFeederHostConfig(FeederHostConfigEditor)
  }

  function setPTFeederHostConfig(editor) {

    if(!editor) {
      return;
    }
    FeederHostConfigEditor = editor;
    if (FeederHostConfigData && FeederHostConfigData.length > 0){
      FeederHostConfigEditor.session.setValue(FeederHostConfigData, 1);
    }

  }

  return {
    init: init,
    callAPILogs: callAPILogs,
    setAPILogsContent: setAPILogsContent,
    callPTFeederLogs: callPTFeederLogs,
    setPTFeederContent: setPTFeederContent,
    callPTFeederAppConfig: callPTFeederAppConfig,
    setPTFeederAppConfig: setPTFeederAppConfig,
    callPTFeederHostConfig: callPTFeederHostConfig,
    setPTFeederHostConfig: setPTFeederHostConfig,
  };
})();
